@@ -60,13 +60,6 @@ def is_file(file_path: str):
     return True
 
 
-class Singleton(object):
-    _instance = None
-    def __new__(cls, *args, **kwargs):
-        if cls._instance is None:
-            cls._instance = object.__new__(cls)
-
-        return cls._instance
 
 
 
